/*
   SHAPESHIFTER DIY OPEN SOURCE DRUM MACHINE
   by FASELUNARE.COM

   https://github.com/faselunare/shapeshifter
   Released under the GPLv3 license.

  Author: Arturo Guadalupi
*/

#include "SSAudio.h"
#include "../LEDs/SSLEDs.h"
#include <Encoder.h>

AudioPlaySdWav           SDtracks[NUMBER_OF_AUDIOTRACKS] = {AudioPlaySdWav(), AudioPlaySdWav(), AudioPlaySdWav(), AudioPlaySdWav()};
AudioMixer4              mixABTrack;      //xy=322,186
AudioMixer4              mixCDTrack;      //xy=325,286
AudioSynthWaveformDc     dc1;            //xy=520,309
AudioMixer4              source_mixer;   //xy=546,240
AudioEffectMultiply      multiply1;      //xy=706,291
AudioFilterBiquad        filter1;        //xy=870,299
AudioEffectBitcrusher    bitcrusher1;    //xy=1060,297
AudioOutputAnalog        dacs;           //xy=1234,297
AudioConnection          patchCord1(SDtracks[0], 0, mixABTrack, 0);
AudioConnection          patchCord2(SDtracks[0], 1, mixABTrack, 1);
AudioConnection          patchCord3(SDtracks[1], 0, mixABTrack, 2);
AudioConnection          patchCord4(SDtracks[1], 1, mixABTrack, 3);
AudioConnection          patchCord5(SDtracks[2], 0, mixCDTrack, 0);
AudioConnection          patchCord6(SDtracks[2], 1, mixCDTrack, 1);
AudioConnection          patchCord7(SDtracks[3], 0, mixCDTrack, 2);
AudioConnection          patchCord8(SDtracks[3], 1, mixCDTrack, 3);
AudioConnection          patchCord9(mixABTrack, 0, source_mixer, 0);
AudioConnection          patchCord10(mixCDTrack, 0, source_mixer, 1);
AudioConnection          patchCord11(dc1, 0, multiply1, 1);
AudioConnection          patchCord12(source_mixer, 0, multiply1, 0);
AudioConnection          patchCord13(multiply1, filter1);
AudioConnection          patchCord14(filter1, bitcrusher1);
AudioConnection          patchCord15(bitcrusher1, 0, dacs, 1);
AudioConnection          patchCord16(bitcrusher1, 0, dacs, 0);

Encoder myEncoder(ENC_PIN2, ENC_PIN1);

//#define AUDIO_DEBUG
void SSAudioClass::begin() {

  tempoBPM = DEFAULT_TEMPO;
  newBPM = DEFAULT_TEMPO;
  myEncoder.write(DEFAULT_TEMPO);
  noteSpeed = 1;

  SPI.setMOSI(SDCARD_MOSI_PIN);
  SPI.setSCK(SDCARD_SCK_PIN);
  SPI.setMISO(SDCARD_MISO_PIN);
  pinMode(LED_BUILTIN, OUTPUT);
  pinMode(SDCARD_DET_PIN, INPUT_PULLUP);

  do {
    if (digitalRead(SDCARD_DET_PIN)) {
#ifdef AUDIO_DEBUG
      Serial.println("SD Not inserted");
#endif
    }
    else {
#ifdef AUDIO_DEBUG
      Serial.println("SD Error");
#endif
    }
    digitalWrite(LED_BUILTIN, HIGH);
    delay(500);
    digitalWrite(LED_BUILTIN, LOW);
    delay(500);
  } while (!SD.begin(SDCARD_CS_PIN));

  digitalWrite(LED_BUILTIN, LOW);
#ifdef AUDIO_DEBUG
  Serial.println("SD OK");
#endif

  AudioMemory(250);
  AudioNoInterrupts();

  filter1.setLowpass(0, 800, 0.54);
  filter1.setLowpass(1, 800, 1.3);
  filter1.setLowpass(2, 800, 0.54);
  filter1.setLowpass(3, 800, 1.3);

  for (int i = 0; i < NUMBER_OF_AUDIOTRACKS; i++) {
    mixerGain[i] = 0.5;
  }

  for (int i = 0; i < NUMBER_OF_CVTRACKS; i++) {
    pinMode(cvPins[i], OUTPUT);
  }

  for (int i = 0; i < NUMBER_OF_TRACKS; i++) {
    currentSteps[i] = 255;
  }

  // BitCrusher
  int currentSampleRate = 22050; // this defaults to passthrough.
  bitcrusher1.bits(NUMBER_OF_STEPS); //set the crusher to defaults. This will passthrough clean at 16,44100
  bitcrusher1.sampleRate(currentSampleRate); //

  for (int i = 0; i < NUMBER_OF_AUDIOTRACKS; i++) {
    source_mixer.gain(i, mixerGain[i]);
    source_mixer.gain(i, mixerGain[i]);
    source_mixer.gain(i, mixerGain[i]);
    source_mixer.gain(i, mixerGain[i]);

    mixABTrack.gain(i, mixerGain[i]);
    mixABTrack.gain(i, mixerGain[i]);
    mixABTrack.gain(i, mixerGain[i]);
    mixABTrack.gain(i, mixerGain[i]);

    mixCDTrack.gain(i, mixerGain[i]);
    mixCDTrack.gain(i, mixerGain[i]);
    mixCDTrack.gain(i, mixerGain[i]);
    mixCDTrack.gain(i, mixerGain[i]);
  }
  dc1.amplitude(0.7);
  AudioInterrupts();
}

void SSAudioClass::run(struct trackInfo *tracks, uint8_t currentTrack) {

  newBPM = myEncoder.read();

  if (newBPM != tempoBPM) {
    tempoBPM = newBPM;

    if (tempoBPM < MIN_TEMPO) {
      tempoBPM = MIN_TEMPO;
      myEncoder.write(MIN_TEMPO);
    }
    else if (tempoBPM > MAX_TEMPO) {
      tempoBPM = MAX_TEMPO;
      myEncoder.write(MAX_TEMPO);
    }
  }
  for (int i = 0; i < NUMBER_OF_TRACKS; i++) {
    localTracks[i].currentPattern = tracks[i].currentPattern;
    for (int j = 0; j < NUMBER_OF_SUBPATTERNS; j++) {
      localTracks[i].pattern[j] = tracks[i].pattern[j];
      localTracks[i].length[j] = tracks[i].length[j];
      localTracks[i].sample[j] = tracks[i].sample[j];
    }
  }

  localCurrentTrack = currentTrack;
  if (playStatus) {
    updateTiming();
  }
  else {
    updateTime = millis();
  }

}

bool SSAudioClass::isPlaying() {
  return playStatus;
}

void SSAudioClass::play() {
  playStatus = true;
}

void SSAudioClass::pause() {
  playStatus = false;
}

void SSAudioClass::updateTiming() {
  if ((millis() - updateTime) > (unsigned long) (tempo)) {
    updateTime = millis();
    updateStep();
  }
}

void SSAudioClass::updateStep() {
#ifdef AUDIO_DEBUG
  Serial.print("Track: ");
  Serial.print(localCurrentTrack + 1);
  Serial.print(" Step: ");
Serial.println(currentSteps[localCurrentTrack]); 300 : 200 = bpm : x     x =
#endif

        //update the steps for each track
  for (int i = 0; i < NUMBER_OF_TRACKS; i++) {
    if (currentSteps[i] < localTracks[i].length[localTracks[i].currentPattern] - 1) {
      currentSteps[i]++;
    }
    else {
      currentSteps[i] = 0;
      if (i < NUMBER_OF_AUDIOTRACKS) {
        SDtracks[i].stop();
      }
    }
  }

  playSample();
  playCV();

  if (currentSteps[localCurrentTrack] % 4 == 0) {
    SSLEDs.functionOn(FUNCTION_PLAY);
  } else {
    SSLEDs.functionOff(FUNCTION_PLAY);
  }

  tempo = (60000 / tempoBPM) / noteSpeed;

  lowPass = analogRead(LOW_PASS_POT);
  if (lowPass != lowPassPrevious) {
    lowPassPrevious = lowPass;
    filter1.setLowpass(0, (lowPass * 20) + 10, 0.54);
    filter1.setLowpass(1, (lowPass * 20) + 10, 1.3);
    filter1.setLowpass(2, (lowPass * 20) + 10, 0.54);
    filter1.setLowpass(3, (lowPass * 20) + 10, 1.3);
  }

  bitCrush = analogRead(BIT_CRUSH_POT);
  if (bitCrush != bitCrushPrevious) {
    bitCrushPrevious = bitCrush;
    bitcrusher1.bits(map(bitCrush, 0, 1023, 0, NUMBER_OF_STEPS));
  }
}

void SSAudioClass::playSample() {
  for (int i = 0; i < NUMBER_OF_AUDIOTRACKS; i++) {
    if (bitRead(localTracks[i].pattern[localTracks[i].currentPattern], currentSteps[i])) {
      selectTracksSamples(i);
    }
  }
}

void SSAudioClass::playCV() {
  for (int i = NUMBER_OF_AUDIOTRACKS; i < NUMBER_OF_TRACKS; i++) {
    if (bitRead(localTracks[i].pattern[localTracks[i].currentPattern], currentSteps[i])) {
      SSLEDs.cvOn(i - NUMBER_OF_AUDIOTRACKS); //scan will show the actual LED
      digitalWriteFast(cvPins[i - NUMBER_OF_AUDIOTRACKS], HIGH);
      delay(5);
    }
    else {
      SSLEDs.cvOff(i - NUMBER_OF_AUDIOTRACKS); // scan will show the actual LED
    }
  }

  for (int i = NUMBER_OF_AUDIOTRACKS; i < NUMBER_OF_TRACKS; i++) {
    if (bitRead(localTracks[i].pattern[localTracks[i].currentPattern], currentSteps[i])) {
      digitalWriteFast(cvPins[i - NUMBER_OF_AUDIOTRACKS], LOW);
    }
  }
}

void SSAudioClass::selectTracksSamples(uint8_t trackNumber) {
  String filename = "tr" + String(trackNumber + 1) + "_" + String(localTracks[trackNumber].sample[localTracks[trackNumber].currentPattern] + 1) + ".wav";
#ifdef AUDIO_DEBUG
  Serial.print("PLAY SAMPLE " + filename + "\n");
#endif
  SDtracks[trackNumber].play(filename.c_str());
}

uint8_t SSAudioClass::getCurrentStep(uint8_t trackNumber) {
  return currentSteps[trackNumber];
}

uint16_t SSAudioClass::getTempo() {
  return  tempoBPM;
}

uint8_t SSAudioClass::getNoteSpeed() {
  return noteSpeed;
}

void SSAudioClass::setNoteSpeed(uint8_t speed) {
  noteSpeed = speed;
}

void SSAudioClass::resetCount() {
  for (int i = 0; i < NUMBER_OF_TRACKS; i++) {
    currentSteps[i] = 0;
  }
}

SSAudioClass SSAudio;
