/*
   SHAPESHIFTER DIY OPEN SOURCE DRUM MACHINE
   by FASELUNARE.COM

   https://github.com/faselunare/shapeshifter
   Released under the GPLv3 license.

  Author: Arturo Guadalupi
*/

#ifndef SSAUDIO_H
#define SSAUDIO_H

#include <Audio.h>
#include "Arduino.h"
#include <SD.h>
#include <SPI.h>
#include <SerialFlash.h>
#include "../../constants.h"
#include "../../pinAssignment.h"
#include "../../structures.h"

class SSAudioClass {
  public:
    void begin();
    bool isPlaying();

    void run(struct trackInfo *tracks, uint8_t currentTrack);
    void play();
    void pause();

    uint8_t getCurrentStep(uint8_t trackNumber);

    uint16_t getTempo();

    uint8_t getNoteSpeed();
    void setNoteSpeed(uint8_t speed);

    void resetCount();

  private:
    float mixerGain[NUMBER_OF_SUBPATTERNS];
    bool playStatus = false;

    //potentiometers
    uint16_t tempo, tempoBPM, newBPM;
    uint16_t lowPass, lowPassPrevious;
    uint16_t bitCrush, bitCrushPrevious;

    unsigned long updateTime;

    void selectTracksSamples(uint8_t trackNumber);
    void playSample();
    void playCV();
    void updateStep();
    void updateTiming();

    struct trackInfo localTracks[NUMBER_OF_TRACKS];
    uint8_t localCurrentTrack;

    uint8_t currentSteps[NUMBER_OF_TRACKS];

    uint8_t noteSpeed;
};

extern SSAudioClass SSAudio;

#endif
