/*
   SHAPESHIFTER DIY OPEN SOURCE DRUM MACHINE
   by FASELUNARE.COM

   https://github.com/faselunare/shapeshifter
   Released under the GPLv3 license.

  Author: Arturo Guadalupi
*/

#include "SSDisplay.h"

#define DISPLAY_DEBUG

void SSDisplayClass::begin() {
  if (!display.begin(SSD1306_SWITCHCAPVCC, 0x3C)) { // Address 0x3C for 128x32
#ifdef DISPLAY_DEBUG
    Serial.println(F("SSD1306 allocation failed"));
#endif
    displayConnected = false;
  }
  else {
    display.clearDisplay();
    display.drawBitmap(0, 0, faseLunareLogo, SCREEN_WIDTH, SCREEN_HEIGHT, 1);
    display.setTextSize(1);
    display.setTextColor(WHITE);
    display.setCursor(10, 25);
    display.println(F("ShapeShifter V2.0"));
    display.display();

    displayConnected = true;
  }
}

bool SSDisplayClass::isConnected() {
  return displayConnected;
}

void SSDisplayClass::printInfo(uint16_t thisTempo, uint8_t thisTrack, uint8_t thisSample, uint8_t thisLength, uint8_t speed, uint8_t subpattern) {
  display.clearDisplay();
  display.setCursor(0, 0);
  display.setTextSize(1);
  display.println("Track " + String(thisTrack + 1) + "  BPM: " + String(thisTempo));
  if (thisTrack < NUMBER_OF_AUDIOTRACKS) {
    display.print("Sample " + String(thisSample + 1) + "    1/");
  }
  else {
    display.print("Sample N/A    1/");
  }
  switch (speed) {
    case 1: display.println("4"); break;
    case 2: display.println("8"); break;
    default: display.println("16"); break;
  }
  display.println("Length  " + String(thisLength));
  display.println("Subpattern  " + String(subpattern));
  display.display();
}

void SSDisplayClass::trackSelection(uint8_t track) {
  display.clearDisplay();
  display.setCursor(0, 0);
  display.setTextSize(2);
  display.println("Track " + String(track + 1));
  display.display();
}

void SSDisplayClass::sampleSelection(uint8_t sample, uint8_t track) {
  display.clearDisplay();
  display.setCursor(0, 0);
  display.setTextSize(2);
  if (track < NUMBER_OF_AUDIOTRACKS) {
    display.println("Sample " + String(sample + 1));
  }
  else {
    display.println("Sample N/A");
  }
  display.display();
}

void SSDisplayClass::lengthSelection(uint8_t length) {
  display.clearDisplay();
  display.setCursor(0, 0);
  display.setTextSize(2);
  display.println("Length " + String(length));
  display.display();
}

void SSDisplayClass::speedSelection(uint8_t speed) {
  display.clearDisplay();
  display.setCursor(0, 0);
  display.setTextSize(2);
  display.println("Speed");
  display.print("1/");
  switch (speed) {
    case 1: display.println("4"); break;
    case 2: display.println("8"); break;
    default: display.println("16"); break;
  }
  display.display();
}

void SSDisplayClass::patternSelection(uint8_t pattern) {
  display.clearDisplay();
  display.setCursor(0, 0);
  display.setTextSize(2);
  display.println("Subpattern");
  display.println(pattern);
  display.display();
}

void SSDisplayClass::saving(void) {
  display.clearDisplay();
  display.setCursor(0, 0);
  display.setTextSize(2);
  display.println("Saving...");
  display.display();
}

void SSDisplayClass::loading(void) {
  display.clearDisplay();
  display.setCursor(0, 0);
  display.setTextSize(2);
  display.println("Loading...");
  display.display();
}

void SSDisplayClass::done(void) {
  display.clearDisplay();
  display.setCursor(0, 0);
  display.setTextSize(3);
  display.println("DONE!");
  display.display();
}

SSDisplayClass SSDisplay;
