/*
   SHAPESHIFTER DIY OPEN SOURCE DRUM MACHINE
   by FASELUNARE.COM

   https://github.com/faselunare/shapeshifter
   Released under the GPLv3 license.

  Author: Arturo Guadalupi
*/

#include "SSLEDs.h"

void SSLEDsClass::begin(int brightness) {
  LEDs.setBrightness(brightness);
  LEDs.begin();
  LEDsMatrix.setBrightness(brightness / 2);
  LEDsMatrix.begin();
  LEDsMatrix.show();
  LEDsMatrix.show();
  LEDs.show();
  LEDs.show();
}

void SSLEDsClass::patternOn(uint8_t track, uint16_t pattern, bool show) {

  for (int i = 0; i < NUMBER_OF_STEPS; i++) {
    if (bitRead(pattern, i)) {
      LEDs.setPixelColor(i, LEDs.Color(tracksColor[track][0], tracksColor[track][1], tracksColor[track][2]));
    }
    else {
      LEDs.setPixelColor(i, 0);
    }
  }
  if (show) {
    LEDs.show();
  }
}

void SSLEDsClass::functionOn(uint8_t thisFunction) {
  if (thisFunction <= FUNCTION_E) {
    LEDs.setPixelColor(thisFunction + FUNCTION_A_LED, LEDs.Color(functionsColor[thisFunction][0], functionsColor[thisFunction][1], functionsColor[thisFunction][2]));
  }
  else {
    if (thisFunction == FUNCTION_SAVE) {
      LEDs.setPixelColor(FUNCTION_SAVE_LED, LEDs.Color(functionsColor[thisFunction][0], functionsColor[thisFunction][1], functionsColor[thisFunction][2]));
    }
    else if (thisFunction == FUNCTION_LOAD) {
      LEDs.setPixelColor(FUNCTION_LOAD_LED, LEDs.Color(functionsColor[thisFunction][0], functionsColor[thisFunction][1], functionsColor[thisFunction][2]));
    }
  }
  LEDs.show();
}

void SSLEDsClass::on(uint8_t thisLED) {
  LEDs.setPixelColor(thisLED, LEDs.Color(0, 0, 255));
}

void SSLEDsClass::off(uint8_t thisLED) {
  LEDs.setPixelColor(thisLED, 0);
}

void SSLEDsClass::cvOn(uint8_t thisCV) {
  LEDs.setPixelColor(thisCV + CV1_LED, LEDs.Color(tracksColor[thisCV + NUMBER_OF_AUDIOTRACKS][0], tracksColor[thisCV + NUMBER_OF_AUDIOTRACKS][1], tracksColor[thisCV + NUMBER_OF_AUDIOTRACKS][2]));
}

void SSLEDsClass::cvOff(uint8_t thisCV) {
  LEDs.setPixelColor(thisCV + CV1_LED, 0);
}

void SSLEDsClass::functionOff(uint8_t thisFunction) {
  if (thisFunction <= FUNCTION_E) {
    LEDs.setPixelColor(thisFunction + FUNCTION_A_LED, 0);
  }
  else {
    if (thisFunction == FUNCTION_SAVE) {
      LEDs.setPixelColor(FUNCTION_SAVE_LED, 0);
    }
    else if (thisFunction == FUNCTION_LOAD) {
      LEDs.setPixelColor(FUNCTION_LOAD_LED, 0);
    }
  }
  LEDs.show();
}

void SSLEDsClass::lengthOn(uint8_t track, uint8_t length) {
  lengthOff();
  for (int i = 0; i < length; i++) {
    LEDs.setPixelColor(i, LEDs.Color(tracksColor[track][0], tracksColor[track][1], tracksColor[track][2]));
  }
  LEDs.show();
}

void SSLEDsClass::lengthOff() {

  for (int i = 0; i < NUMBER_OF_STEPS; i++) {
    LEDs.setPixelColor(i, 0);
  }
  LEDs.show();
}

void SSLEDsClass::setFunctionColor(uint8_t functionButton, uint8_t red, uint8_t green, uint8_t blue) {
  functionsColor[functionButton][0] = red;
  functionsColor[functionButton][1] = green;
  functionsColor[functionButton][2] = blue;
}

void SSLEDsClass::setTrackColor(uint8_t track, uint8_t red, uint8_t green, uint8_t blue) {
  tracksColor[track - 1][0] = red;
  tracksColor[track - 1][1] = green;
  tracksColor[track - 1][2] = blue;
}

void SSLEDsClass::scan(uint8_t track, uint8_t show) {
  for (int i = 0; i < NUMBER_OF_TRACKS; i++) {
    currentStep[i] = SSAudio.getCurrentStep(i);
  }
  if (show != FUNCTION_E) {
    if (currentStep[track] != previousStep[track]) {
      SSLEDs.patternOn(track, SSPattern.getPattern(), false); //show LEDs
      LEDs.setPixelColor(currentStep[track], LEDs.Color(255, 255, 255));
      LEDs.show();
    }
  }

  for (int i = 0; i < NUMBER_OF_AUDIOTRACKS; i++) {
    if (currentStep[i] != previousStep[i]) {
      LEDsMatrix.setPixelColor((previousStep[i] * 4) + i, LEDsMatrix.Color(tracksColor[i][0], tracksColor[i][1], tracksColor[i][2]));
      LEDsMatrix.setPixelColor((currentStep[i] * 4) + i, LEDsMatrix.Color(255, 255, 255));
      LEDsMatrix.show();
    }
  }

  for (int i = 0; i < NUMBER_OF_TRACKS; i++) {
    if (currentStep[i] != previousStep[i]) {
      previousStep[i] = currentStep[i];
    }
  }
}

void SSLEDsClass::showLengths(struct trackInfo *tracks) {
  for (int i = 0; i < MATRIX_LENGTH; i++) {
    LEDsMatrix.setPixelColor(i, 0);
    for (int j = 0; j < NUMBER_OF_AUDIOTRACKS; j++) {
      for (int k = 0; k < tracks[j].length[tracks[j].currentPattern]; k++) {
        LEDsMatrix.setPixelColor((k * 4) + j, LEDs.Color(functionsColor[j][0], functionsColor[j][1], functionsColor[j][2]));
      }
    }
  }

  LEDsMatrix.show();
}

SSLEDsClass SSLEDs;
