/*
   SHAPESHIFTER DIY OPEN SOURCE DRUM MACHINE
   by FASELUNARE.COM

   https://github.com/faselunare/shapeshifter
   Released under the GPLv3 license.

  Author: Arturo Guadalupi
*/

#ifndef SSLEDS_H
#define SSLEDS_H

#include "Arduino.h"
#include "../../constants.h"
#include "../../structures.h"
#include "../../pinAssignment.h"
//#include <Adafruit_NeoPixel.h>
#include <WS2812Serial.h>
#include "../Audio/SSAudio.h"
#include "../Pattern/SSPattern.h"

#define FUNCTION_A_LED          16
#define FUNCTION_B_LED          17
#define FUNCTION_C_LED          18
#define FUNCTION_D_LED          19
#define FUNCTION_E_LED          20
#define FUNCTION_PLAY_LED       21
#define FUNCTION_SAVE_LED       33
#define FUNCTION_LOAD_LED       34

#define CLKIN_LED               26
#define CLKOUT_LED              27
#define MASTERRST_LED           28

#define CV1_LED                 22
#define CV2_LED                 23
#define CV3_LED                 24
#define CV4_LED                 25



class SSLEDsClass {
  public:
    void begin(int brightness);

    void patternOn(uint8_t track, uint16_t pattern, bool show = true);

    void functionOn(uint8_t thisFunction);
    void functionOff(uint8_t thisFunction);

    void on(uint8_t thisLED);
    void off(uint8_t thisLED);

    void cvOn(uint8_t thisCv);
    void cvOff(uint8_t thisFunction);

    void lengthOn(uint8_t track, uint8_t length);
    void lengthOff();

    void setFunctionColor(uint8_t functionButton, uint8_t red, uint8_t green, uint8_t blue);
    void setTrackColor(uint8_t track, uint8_t red, uint8_t green, uint8_t blue);

    void scan(uint8_t track, uint8_t show);

    void showLengths(struct trackInfo *tracks);

  private:


    byte drawingMemory[LEDs_LENGTH*3];         //  3 bytes per LED
     byte displayMemory[LEDs_LENGTH*12]; // 12 bytes per LED

    byte MatrixDrawingMemory[MATRIX_LENGTH*3];         //  3 bytes per LED
     byte MatrixDisplayMemory[MATRIX_LENGTH*12]; // 12 bytes per LED

    //Adafruit_NeoPixel LEDs = Adafruit_NeoPixel(LEDs_LENGTH, LEDs_PIN, NEO_GRB + NEO_KHZ800);
    //Adafruit_NeoPixel LEDsMatrix = Adafruit_NeoPixel(MATRIX_LENGTH, MATRIX_PIN, NEO_GRB + NEO_KHZ800);
    WS2812Serial LEDs(LEDs_LENGTH, displayMemory, drawingMemory, LEDs_PIN, WS2812_GRB);
    WS2812Serial LEDsMatrix(MATRIX_LENGTH, MatrixDisplayMemory, MatrixDrawingMemory, MATRIX_PIN, WS2812_GRB);

    uint8_t functionsColor[NUMBER_OF_FUNCTIONS_LEDS][NUMBER_OF_COLORS];
    uint8_t tracksColor[NUMBER_OF_TRACKS][NUMBER_OF_COLORS];

    uint8_t previousStep[NUMBER_OF_TRACKS], currentStep[NUMBER_OF_TRACKS];
};

extern SSLEDsClass SSLEDs;

#endif
