/*
   SHAPESHIFTER DIY OPEN SOURCE DRUM MACHINE
   by FASELUNARE.COM

   https://github.com/faselunare/shapeshifter
   Released under the GPLv3 license.

  Author: Arturo Guadalupi
*/

#include "SSMemory.h"
#define MEMORY_DEBUG

File myFile;
#define JSON_CAPACITY 13*JSON_ARRAY_SIZE(4) + JSON_OBJECT_SIZE(1) + 4*JSON_OBJECT_SIZE(4) + 1500

void SSMemoryClass::save(struct trackInfo *tracks, uint8_t memorySlot) {
  String filename = String(memorySlot) + "_J";

  if (SD.exists(filename.c_str())) {
    SD.remove(filename.c_str());
#ifdef MEMORY_DEBUG
    Serial.println("Removing: " + filename);
#endif
  }

  myFile = SD.open(filename.c_str(), FILE_WRITE);

  if (myFile) {
    const size_t capacity = JSON_CAPACITY;
    DynamicJsonDocument doc(capacity);

    JsonArray jsonTracks = doc.createNestedArray("tracks");

    for (int i = 0; i < NUMBER_OF_TRACKS; i++) {
      JsonObject track_i = jsonTracks.createNestedObject();
      track_i["currentPattern"] = tracks[i].currentPattern;
      JsonArray track_i_pattern_j = track_i.createNestedArray("pattern");
      JsonArray track_i_length_j = track_i.createNestedArray("length");
      JsonArray track_i_sample_j = track_i.createNestedArray("sample");
      for (int j = 0; j < NUMBER_OF_SUBPATTERNS; j++) {
        track_i_pattern_j.add(tracks[i].pattern[j]);
        track_i_length_j.add(tracks[i].length[j]);
        track_i_sample_j.add(tracks[i].sample[j] + 1);
      }
    }

    serializeJsonPretty(doc, myFile);
#ifdef MEMORY_DEBUG
    serializeJsonPretty(doc, Serial);
#endif
    myFile.close();
  }
}

void SSMemoryClass::load(struct trackInfo *tracks, uint8_t memorySlot) {
  String filename = String(memorySlot) + "_J";
  myFile = SD.open(filename.c_str(), FILE_READ);

  if (myFile) {
    const size_t capacity = JSON_CAPACITY;
    DynamicJsonDocument doc(capacity);
    deserializeJson(doc, myFile);
#ifdef MEMORY_DEBUG
    serializeJsonPretty(doc, Serial);
#endif

    JsonArray jsonTracks = doc["tracks"];

    for (int i = 0; i < NUMBER_OF_TRACKS; i++) {
      JsonObject tracks_i = jsonTracks[i];
      int tracks_i_currentPattern = tracks_i["currentPattern"];
      tracks[i].currentPattern = tracks_i_currentPattern;

      JsonArray tracks_i_pattern_j = tracks_i["pattern"];
      JsonArray tracks_i_length_j = tracks_i["length"];
      JsonArray tracks_i_sample_j = tracks_i["sample"];
      for (int j = 0; j < NUMBER_OF_SUBPATTERNS; j++) {
        int tracks_i_pattern = tracks_i_pattern_j[j];
        tracks[i].pattern[j] = tracks_i_pattern;

        int tracks_i_length = tracks_i_length_j[j];
        tracks[i].length[j] = tracks_i_length;

        int tracks_i_sample = tracks_i_sample_j[j];
        tracks[i].sample[j] = tracks_i_sample - 1;
      }
    }
  }
  else {
#ifdef MEMORY_DEBUG
    Serial.println("FILE NOT FOUND!");
#endif
  }
}

SSMemoryClass SSMemory;
